@@ -6,7 +6,7 @@ use warnings;
 
 use Padre::Wx ();
 
-our $VERSION = '0.45';
+our $VERSION = '0.46';
 our @ISA     = 'Padre::Document';
 
 # max lines to display in a calltip
@@ -185,7 +185,12 @@ sub keywords {
 	return $self->{keywords};
 }
 
-sub comment_lines_str { return '#' }
+# In Perl 6 the best way to comment the current error reliably is 
+# by putting a hash and a space since #( is an embedded comment in Perl 6!
+# see S02:166
+sub comment_lines_str { 
+	return '# ';
+}
 
 #
 # Guess the new line for the current document
@@ -215,13 +220,15 @@ sub guess_newline {
 sub _find_quick_fix {
 	my ($self, $editor) = @_;
 	
-	my $new_line = $self->guess_newline;
+	if(not defined $self->{issues}) {
+		$self->{issues} = [];
+	}
+
+	my $nl = $self->guess_newline;
 	my $current_line_no = $editor->GetCurrentLine;
 	
 	my @items = ();
 	my $num_issues = scalar @{$self->{issues}};
-	print "Number of issues: $num_issues\n";
-	my $comment_error_added = 0;
 	foreach my $issue ( @{$self->{issues}} ) {
 		my $issue_line_no = $issue->{line} - 1;
 		if($issue_line_no == $current_line_no) {
@@ -231,12 +238,24 @@ sub _find_quick_fix {
 				
 				my $var_name = $1;
 
+				# Fixes the following:
+				# 	$foo = 1;
+				# into:
+				# 	my $foo;
+				#	$foo = 1;
 				push @items, {
 					text     => sprintf( Wx::gettext('Insert declaration for %s'), $var_name),
 					listener => sub { 
 						#Insert a variable declaration before the start of the current line
 						my $line_start = $editor->PositionFromLine( $current_line_no );
-						$editor->InsertText($line_start, "my $var_name;$new_line");
+						my $line_end   = $editor->GetLineEndPosition( $current_line_no );
+						my $line_text  = $editor->GetTextRange($line_start, $line_end);
+						my $indent = ($line_text =~ /(^\s+)/) ? $1 : '';
+						$line_text = 
+							"${indent}my $var_name;$nl" .
+							$line_text;
+						$editor->SetSelection( $line_start, $line_end );
+						$editor->ReplaceSelection( $line_text );
 					},
 				};
 			
@@ -250,6 +269,10 @@ sub _find_quick_fix {
 				);
 				foreach my $keyword (@flow_control_keywords) {
 					if($keyword eq $routine_name) {
+						# Fixes the following:
+						# 	if() { }; 
+						# into:
+						# 	if () { };
 						push @items, {
 							text     => sprintf( Wx::gettext('Insert a space after %s'), $keyword ),
 							listener => sub { 
@@ -266,18 +289,36 @@ sub _find_quick_fix {
 						last;
 					}
 				}
+				# Fixes the following:
+				# 	foo();
+				# into:
+				# 	sub foo() {
+				#		#XXX-implement
+				# 	}
+				# 	foo();
 				push @items, {
 					text     => sprintf( Wx::gettext('Insert routine %s'), $routine_name),
 					listener => sub { 
 						#Insert an empty routine definition before the current line
 						my $line_start = $editor->PositionFromLine( $current_line_no );
-						$editor->InsertText($line_start, 
-							"sub $routine_name {$new_line\t#XXX-implement$new_line}$new_line");
+						my $line_end   = $editor->GetLineEndPosition( $current_line_no );
+						my $line_text  = $editor->GetTextRange($line_start, $line_end);
+						my $indent = ($line_text =~ /(^\s+)/) ? $1 : '';
+						$line_text =  "${indent}sub $routine_name {$nl" .
+							"${indent}\t#XXX-implement$nl" .
+							"${indent}}$nl" .
+							$line_text;
+						$editor->SetSelection( $line_start, $line_end );
+						$editor->ReplaceSelection( $line_text );							
 					},
 				};
 			
 			} elsif($issue_msg =~ /^Obsolete use of . to concatenate strings/i) {
 
+				# Fixes the following:
+				# 	$string = "a" . "b";
+				# into:
+				# 	$string = "a" ~ "b";
 				push @items, {
 					text     => Wx::gettext('Use ~ instead of . for string concatenation'),
 					listener => sub { 
@@ -293,6 +334,10 @@ sub _find_quick_fix {
 			
 			} elsif($issue_msg =~ /^Obsolete use of -> to call a method/i) {
 
+				# Fixes the following:
+				# 	P->foo;
+				# into:
+				# 	P.foo;
 				push @items, {
 					text     => Wx::gettext('Use . for method call'),
 					listener => sub { 
@@ -308,6 +353,10 @@ sub _find_quick_fix {
 			
 			} elsif($issue_msg =~ /^Obsolete use of C\+\+ constructor syntax/i) {
 
+				# Fixes the following:
+				# 	new Foo;
+				# into:
+				# 	Foo.new;
 				push @items, {
 					text     => Wx::gettext('Use Perl 6 constructor syntax'),
 					listener => sub { 
@@ -324,6 +373,10 @@ sub _find_quick_fix {
 			
 			} elsif($issue_msg =~ /^Obsolete use of C-style "for \(;;\)" loop/i) {
 
+				# Fixes the following:
+				# 	for(;;) { };
+				# into:
+				# 	loop(;;) { };
 				push @items, {
 					text     => Wx::gettext('Use loop (;;) for looping'),
 					listener => sub { 
@@ -339,6 +392,10 @@ sub _find_quick_fix {
 				
 			} elsif($issue_msg =~ /^Obsolete use of \[-1\] subscript to access final element/i) {
 
+				# Fixes the following:
+				# 	[-1];
+				# into:
+				# 	[*-1];
 				push @items, {
 					text     => Wx::gettext('Use [*-1] to access final element'),
 					listener => sub { 
@@ -354,6 +411,10 @@ sub _find_quick_fix {
 			
 			} elsif($issue_msg =~ /^Obsolete use of rand\(N\)/i) {
 			
+				# Fixes the following:
+				# 	rand(10);
+				# into:
+				# 	10.pick;
 				push @items, {
 					text     => Wx::gettext('Use N.pick for a random number'),
 					listener => sub { 
@@ -367,6 +428,10 @@ sub _find_quick_fix {
 					},
 				};
 
+				# Fixes the following:
+				# 	rand(10);
+				# into:
+				# 	(1..10).pick;
 				push @items, {
 					text     => Wx::gettext('Use (1..N).pick for a random number'),
 					listener => sub { 
@@ -382,6 +447,10 @@ sub _find_quick_fix {
 				
 			} elsif($issue_msg =~ /^Please use \.\.\* for indefinite range/i) {
 
+				# Fixes the following:
+				# 	[1..];
+				# into:
+				# 	[1..*];
 				push @items, {
 					text     => Wx::gettext('Use [N..*] for indefinite range'),
 					listener => sub { 
@@ -397,6 +466,10 @@ sub _find_quick_fix {
 			
 			} elsif($issue_msg =~ /^Please use \!\! rather than \:\:/i) {
 
+				# Fixes the following:
+				# 	1 == 2 ?? 1 :: 2;
+				# into:
+				# 	1 == 2 ?? 1 !! 2;
 				push @items, {
 					text     => Wx::gettext('Use !! for conditional operator else clause'),
 					listener => sub { 
@@ -413,9 +486,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Precedence too loose within \?\?\!\!/i) {
 
 				# Fixes errors like:
-				# 42 ?? 1,2,3 Z 4,5,6 !! 1,2,3 X 4,5,6;
+				# 	42 ?? 1,2,3 Z 4,5,6 !! 1,2,3 X 4,5,6;
 				# into:
-				# 42 ?? (1,2,3 Z 4,5,6) !! 1,2,3 X 4,5,6;
+				# 	42 ?? (1,2,3 Z 4,5,6) !! 1,2,3 X 4,5,6;
 				push @items, {
 					text     => Wx::gettext('Use ?? (...) !! to avoid precedence bugs'),
 					listener => sub { 
@@ -433,9 +506,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Obsolete use of \?\: for the conditional operator/i) {
 
 				# Fixes the following:
-				# (1 == 1) ? 1 : 2
+				# 	(1 == 1) ? 1 : 2
 				# into:
-				# (1 == 1) ?? 1 !! 2
+				# 	(1 == 1) ?? 1 !! 2
 				push @items, {
 					text     => Wx::gettext('Use ?? !! for the conditional operator'),
 					listener => sub { 
@@ -452,9 +525,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Possible obsolete use of \.\= as append operator/i) {
 
 				# Fixes the following:
-				# $string .= "a";
+				# 	$string .= "a";
 				# into:
-				# $string ~= "a";
+				# 	$string ~= "a";
 				push @items, {
 					text     => Wx::gettext('Use ~= for string concatenation'),
 					listener => sub { 
@@ -471,9 +544,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Obsolete use of \=\~ to do pattern matching/i) {
 
 				# Fixes the following:
-				# $string =~ /abc/;
+				# 	$string =~ /abc/;
 				# into:
-				# $string ~~ /abc/;
+				# 	$string ~~ /abc/;
 				push @items, {
 					text     => Wx::gettext('Use ~~ for pattern matching'),
 					listener => sub { 
@@ -490,9 +563,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Obsolete use of \!\~ to do negated pattern matching/i) {
 
 				# Fixes the following:
-				# $string !~ /abc/;
+				# 	$string !~ /abc/;
 				# into:
-				# $string !~~ /abc/;
+				# 	$string !~~ /abc/;
 				push @items, {
 					text     => Wx::gettext('Use !~~ for negated pattern matching'),
 					listener => sub { 
@@ -509,9 +582,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Obsolete use of >> to do right shift/i) {
 
 				# Fixes the following:
-				# 2 >> 1;
+				# 	2 >> 1;
 				# into:
-				# 2 +> 1;
+				# 	2 +> 1;
 				push @items, {
 					text     => Wx::gettext('Use +> for numeric right shift'),
 					listener => sub { 
@@ -526,9 +599,9 @@ sub _find_quick_fix {
 				};
 			
 				# Fixes the following:
-				# 100 >> 1;
+				# 	100 >> 1;
 				# into:
-				# 100 ~> 1;
+				# 	100 ~> 1;
 				push @items, {
 					text     => Wx::gettext('Use ~> for string right shift'),
 					listener => sub { 
@@ -544,9 +617,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Obsolete use of << to do left shift/i) {
 
 				# Fixes the following:
-				# 2 << 1;
+				# 	2 << 1;
 				# into:
-				# 2 +< 1;
+				# 	2 +< 1;
 				push @items, {
 					text     => Wx::gettext('Use +< for numeric left shift'),
 					listener => sub { 
@@ -561,9 +634,9 @@ sub _find_quick_fix {
 				};
 			
 				# Fixes the following:
-				# 100 << 1;
+				# 	100 << 1;
 				# into:
-				# 100 ~< 1;
+				# 	100 ~< 1;
 				push @items, {
 					text     => Wx::gettext('Use ~< for string left shift'),
 					listener => sub { 
@@ -580,9 +653,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Obsolete use of \$\@ variable as eval error/i) {
 
 				# Fixes the following:
-				# $@;
+				# 	$@;
 				# into:
-				# $!;
+				# 	$!;
 				push @items, {
 					text     => Wx::gettext('Use $! for eval errors'),
 					listener => sub { 
@@ -599,9 +672,9 @@ sub _find_quick_fix {
 			} elsif($issue_msg =~ /^Obsolete use of \$\] variable/i) {
 
 				# Fixes the following:
-				# $];
+				# 	$];
 				# into:
-				# $::PERL_VERSION;
+				# 	$::PERL_VERSION;
 				push @items, {
 					text     => Wx::gettext('Use $::PERL_VERSION'),
 					listener => sub { 
@@ -617,19 +690,77 @@ sub _find_quick_fix {
 			
 			}
 
-			if(not $comment_error_added) {
+		}
+
+	}
+	
+	if($num_issues) {
+		
+		# add "comment error line" as the last resort to solving an issue
+		foreach my $issue ( @{$self->{issues}} ) {
+			my $issue_line_no = $issue->{line} - 1;
+			if($issue_line_no == $current_line_no) {
+				# Fixes the following:
+				# 	some_weird_error();
+				# into:
+				# 	# some_weird_error();
 				push @items, {
 					text     => Wx::gettext('Comment error line'),
 					listener => sub {
 						# comment current error by putting a hash and a space
-						# since #( is an embedded comment in Perl 6!
-						# see S02:166
+						# since #( is an embedded comment in Perl 6! see S02:166
 						my $line_start = $editor->PositionFromLine( $current_line_no );
-						$editor->InsertText($line_start, '# ');
+						my $line_end   = $editor->GetLineEndPosition( $current_line_no );
+						my $line_text  = $editor->GetTextRange($line_start, $line_end);
+						$line_text = "# ${line_text}";
+						$editor->SetSelection( $line_start, $line_end );
+						$editor->ReplaceSelection( $line_text );
 					},
 				};
-				$comment_error_added = 1;
+				last;
 			}
+		}
+		
+	} else {
+
+		# No issues; let us provide a some helpful quick fixes
+		my $selected_text = $editor->GetSelectedText;
+		if($selected_text && $selected_text =~ /[\n\r]/) {
+			
+			# Fixes the following:
+			# 	faulty_code();
+			# into:
+			# 	try {
+			#		faulty_code();
+			#
+			#		CATCH {
+			#			warn "oops: $!";
+			#		}
+			#	}
+
+			push @items, {
+				text     => Wx::gettext('Surround with try { ... }'),
+				listener => sub {
+					# Surround the 'selection' with a try { 'selection'  CATCH { } }
+					my $line_start = $editor->PositionFromLine( 
+						$editor->LineFromPosition($editor->GetSelectionStart) 
+					);
+					my $line_end = $editor->PositionFromLine( 
+						$editor->LineFromPosition($editor->GetSelectionEnd) 
+					);
+					
+					my $indent = ($selected_text =~ /(^\s+)/) ? $1 : '';
+					$selected_text =~ s/^/\t/gm;
+					my $line_text =  "${indent}try {$nl" .
+						"$selected_text$nl" . 
+						"${indent}\tCATCH {$nl" .
+						"${indent}\t\twarn \"oops: \$!\";$nl" .
+						"${indent}\t}$nl" .
+						"${indent}}$nl";
+					$editor->SetSelection( $line_start, $line_end );
+					$editor->ReplaceSelection( $line_text );
+				},
+			};
 			
 		}
 	}
